=== modified file 'lib/lp/archiveuploader/tests/test_uploadprocessor.py'
--- lib/lp/archiveuploader/tests/test_uploadprocessor.py	2011-12-30 06:14:56 +0000
+++ lib/lp/archiveuploader/tests/test_uploadprocessor.py	2012-01-03 12:11:57 +0000
@@ -629,7 +629,8 @@
         from_addr, to_addrs, raw_msg = stub.test_emails.pop()
         foo_bar = "Foo Bar <foo.bar@canonical.com>"
         daniel = "Daniel Silverstone <daniel.silverstone@canonical.com>"
-        self.assertEqual([e.strip() for e in to_addrs], [foo_bar, daniel])
+        self.assertContentEqual(
+            [foo_bar, daniel], [e.strip() for e in to_addrs])
         self.assertTrue(
             "NEW" in raw_msg, "Expected email containing 'NEW', got:\n%s"
             % raw_msg)
@@ -663,7 +664,8 @@
         from_addr, to_addrs, raw_msg = stub.test_emails.pop()
         daniel = "Daniel Silverstone <daniel.silverstone@canonical.com>"
         foo_bar = "Foo Bar <foo.bar@canonical.com>"
-        self.assertEqual([e.strip() for e in to_addrs], [foo_bar, daniel])
+        self.assertContentEqual(
+            [foo_bar, daniel], [e.strip() for e in to_addrs])
         self.assertTrue("Waiting for approval" in raw_msg,
                         "Expected an 'upload awaits approval' email.\n"
                         "Got:\n%s" % raw_msg)

=== modified file 'lib/lp/buildmaster/interfaces/builder.py'
--- lib/lp/buildmaster/interfaces/builder.py	2012-01-01 02:58:52 +0000
+++ lib/lp/buildmaster/interfaces/builder.py	2012-01-03 12:11:57 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 # pylint: disable-msg=E0211,E0213
@@ -195,6 +195,8 @@
 
     def setSlaveForTesting(proxy):
         """Sets the RPC proxy through which to operate the build slave."""
+        # XXX JeroenVermeulen 2011-11-09, bug=888010: Don't use this.
+        # It's a trap.  See bug for details.
 
     def verifySlaveBuildCookie(slave_build_id):
         """Verify that a slave's build cookie is consistent.

=== modified file 'lib/lp/buildmaster/manager.py'
--- lib/lp/buildmaster/manager.py	2012-01-01 02:58:52 +0000
+++ lib/lp/buildmaster/manager.py	2012-01-03 12:17:29 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Soyuz buildd slave manager logic."""
@@ -34,6 +34,7 @@
     BuildBehaviorMismatch,
     )
 from lp.buildmaster.model.builder import Builder
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
 
 
 BUILDD_MANAGER_LOG_NAME = "slave-scanner"
@@ -111,13 +112,17 @@
     # algorithm for polling.
     SCAN_INTERVAL = 15
 
-    def __init__(self, builder_name, logger):
+    def __init__(self, builder_name, logger, clock=None):
         self.builder_name = builder_name
         self.logger = logger
+        if clock is None:
+            clock = reactor
+        self._clock = clock
 
     def startCycle(self):
         """Scan the builder and dispatch to it or deal with failures."""
         self.loop = LoopingCall(self.singleCycle)
+        self.loop.clock = self._clock
         self.stopping_deferred = self.loop.start(self.SCAN_INTERVAL)
         return self.stopping_deferred
 
@@ -138,51 +143,58 @@
         1. Print the error in the log
         2. Increment and assess failure counts on the builder and job.
         """
-        # Make sure that pending database updates are removed as it
-        # could leave the database in an inconsistent state (e.g. The
-        # job says it's running but the buildqueue has no builder set).
+        # Since this is a failure path, we could be in a broken
+        # transaction.  Get us a fresh one.
         transaction.abort()
 
         # If we don't recognise the exception include a stack trace with
         # the error.
         error_message = failure.getErrorMessage()
-        if failure.check(
+        familiar_error = failure.check(
             BuildSlaveFailure, CannotBuild, BuildBehaviorMismatch,
-            CannotResumeHost, BuildDaemonError, CannotFetchFile):
-            self.logger.info("Scanning %s failed with: %s" % (
-                self.builder_name, error_message))
+            CannotResumeHost, BuildDaemonError, CannotFetchFile)
+        if familiar_error:
+            self.logger.info(
+                "Scanning %s failed with: %s",
+                self.builder_name, error_message)
         else:
-            self.logger.info("Scanning %s failed with: %s\n%s" % (
+            self.logger.info(
+                "Scanning %s failed with: %s\n%s",
                 self.builder_name, failure.getErrorMessage(),
-                failure.getTraceback()))
+                failure.getTraceback())
 
         # Decide if we need to terminate the job or fail the
         # builder.
         try:
             builder = get_builder(self.builder_name)
-            builder.gotFailure()
-            if builder.currentjob is not None:
-                build_farm_job = builder.getCurrentBuildFarmJob()
-                build_farm_job.gotFailure()
-                self.logger.info(
-                    "builder %s failure count: %s, "
-                    "job '%s' failure count: %s" % (
+            transaction.commit()
+
+            with DatabaseTransactionPolicy(read_only=False):
+                builder.gotFailure()
+
+                if builder.currentjob is None:
+                    self.logger.info(
+                        "Builder %s failed a probe, count: %s",
+                        self.builder_name, builder.failure_count)
+                else:
+                    build_farm_job = builder.getCurrentBuildFarmJob()
+                    build_farm_job.gotFailure()
+                    self.logger.info(
+                        "builder %s failure count: %s, "
+                        "job '%s' failure count: %s",
                         self.builder_name,
                         builder.failure_count,
                         build_farm_job.title,
-                        build_farm_job.failure_count))
-            else:
-                self.logger.info(
-                    "Builder %s failed a probe, count: %s" % (
-                        self.builder_name, builder.failure_count))
-            assessFailureCounts(builder, failure.getErrorMessage())
-            transaction.commit()
+                        build_farm_job.failure_count)
+
+                assessFailureCounts(builder, failure.getErrorMessage())
+                transaction.commit()
         except:
             # Catastrophic code failure! Not much we can do.
+            transaction.abort()
             self.logger.error(
                 "Miserable failure when trying to examine failure counts:\n",
                 exc_info=True)
-            transaction.abort()
 
     def checkCancellation(self, builder):
         """See if there is a pending cancellation request.
@@ -205,8 +217,9 @@
             return defer.succeed(True)
 
         self.logger.info("Cancelling build '%s'" % build.title)
-        buildqueue.cancel()
-        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            buildqueue.cancel()
+            transaction.commit()
         d = builder.resumeSlaveHost()
         d.addCallback(resume_done)
         return d
@@ -236,14 +249,9 @@
         """
         # We need to re-fetch the builder object on each cycle as the
         # Storm store is invalidated over transaction boundaries.
-
         self.builder = get_builder(self.builder_name)
 
         def status_updated(ignored):
-            # Commit the changes done while possibly rescuing jobs, to
-            # avoid holding table locks.
-            transaction.commit()
-
             # See if we think there's an active build on the builder.
             buildqueue = self.builder.getBuildQueue()
 
@@ -253,14 +261,10 @@
                 return self.builder.updateBuild(buildqueue)
 
         def build_updated(ignored):
-            # Commit changes done while updating the build, to avoid
-            # holding table locks.
-            transaction.commit()
-
             # If the builder is in manual mode, don't dispatch anything.
             if self.builder.manual:
                 self.logger.debug(
-                    '%s is in manual mode, not dispatching.' %
+                    '%s is in manual mode, not dispatching.',
                     self.builder.name)
                 return
 
@@ -278,22 +282,33 @@
                 job = self.builder.currentjob
                 if job is not None and not self.builder.builderok:
                     self.logger.info(
-                        "%s was made unavailable, resetting attached "
-                        "job" % self.builder.name)
-                    job.reset()
+                        "%s was made unavailable; resetting attached job.",
+                        self.builder.name)
                     transaction.commit()
+                    with DatabaseTransactionPolicy(read_only=False):
+                        job.reset()
+                        transaction.commit()
                 return
 
             # See if there is a job we can dispatch to the builder slave.
 
+            # XXX JeroenVermeulen 2011-10-11, bug=872112: The job's
+            # failure count will be reset once the job has started
+            # successfully.  Because of intervening commits, you may see
+            # a build with a nonzero failure count that's actually going
+            # to succeed later (and have a failure count of zero).  Or
+            # it may fail yet end up with a lower failure count than you
+            # saw earlier.
             d = self.builder.findAndStartJob()
 
             def job_started(candidate):
                 if self.builder.currentjob is not None:
                     # After a successful dispatch we can reset the
                     # failure_count.
-                    self.builder.resetFailureCount()
                     transaction.commit()
+                    with DatabaseTransactionPolicy(read_only=False):
+                        self.builder.resetFailureCount()
+                        transaction.commit()
                     return self.builder.slave
                 else:
                     return None
@@ -372,6 +387,7 @@
         self.logger = self._setupLogger()
         self.new_builders_scanner = NewBuildersScanner(
             manager=self, clock=clock)
+        self.transaction_policy = DatabaseTransactionPolicy(read_only=True)
 
     def _setupLogger(self):
         """Set up a 'slave-scanner' logger that redirects to twisted.
@@ -390,16 +406,28 @@
         logger.setLevel(level)
         return logger
 
+    def enterReadOnlyDatabasePolicy(self):
+        """Set the database transaction policy to read-only.
+
+        Any previously pending changes are committed first.
+        """
+        transaction.commit()
+        self.transaction_policy.__enter__()
+
+    def exitReadOnlyDatabasePolicy(self, *args):
+        """Reset database transaction policy to the default read-write."""
+        self.transaction_policy.__exit__(None, None, None)
+
     def startService(self):
         """Service entry point, called when the application starts."""
+        # Avoiding circular imports.
+        from lp.buildmaster.interfaces.builder import IBuilderSet
+
+        self.enterReadOnlyDatabasePolicy()
 
         # Get a list of builders and set up scanners on each one.
-
-        # Avoiding circular imports.
-        from lp.buildmaster.interfaces.builder import IBuilderSet
-        builder_set = getUtility(IBuilderSet)
-        builders = [builder.name for builder in builder_set]
-        self.addScanForBuilders(builders)
+        self.addScanForBuilders(
+            [builder.name for builder in getUtility(IBuilderSet)])
         self.new_builders_scanner.scheduleScan()
 
         # Events will now fire in the SlaveScanner objects to scan each
@@ -420,6 +448,7 @@
         # stopped, so we can wait on them all at once here before
         # exiting.
         d = defer.DeferredList(deferreds, consumeErrors=True)
+        d.addCallback(self.exitReadOnlyDatabasePolicy)
         return d
 
     def addScanForBuilders(self, builders):

=== modified file 'lib/lp/buildmaster/model/builder.py'
--- lib/lp/buildmaster/model/builder.py	2012-01-02 11:21:14 +0000
+++ lib/lp/buildmaster/model/builder.py	2012-01-03 12:11:57 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009,2011 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 # pylint: disable-msg=E0611,W0212
@@ -66,6 +66,7 @@
     SQLBase,
     sqlvalues,
     )
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
 from lp.services.helpers import filenameToContentType
 from lp.services.job.interfaces.job import JobStatus
 from lp.services.job.model.job import Job
@@ -545,6 +546,8 @@
 
     def setSlaveForTesting(self, proxy):
         """See IBuilder."""
+        # XXX JeroenVermeulen 2011-11-09, bug=888010: Don't use this.
+        # It's a trap.  See bug for details.
         self._testing_slave = proxy
         del get_property_cache(self).slave
 
@@ -673,10 +676,13 @@
                 bytes_written = out_file.tell()
                 out_file.seek(0)
 
-                library_file = getUtility(ILibraryFileAliasSet).create(
-                    filename, bytes_written, out_file,
-                    contentType=filenameToContentType(filename),
-                    restricted=private)
+                transaction.commit()
+                with DatabaseTransactionPolicy(read_only=False):
+                    library_file = getUtility(ILibraryFileAliasSet).create(
+                        filename, bytes_written, out_file,
+                        contentType=filenameToContentType(filename),
+                        restricted=private)
+                    transaction.commit()
             finally:
                 # Remove the temporary file.  getFile() closes the file
                 # object.
@@ -714,7 +720,7 @@
     def acquireBuildCandidate(self):
         """Acquire a build candidate in an atomic fashion.
 
-        When retrieiving a candidate we need to mark it as building
+        When retrieving a candidate we need to mark it as building
         immediately so that it is not dispatched by another builder in the
         build manager.
 
@@ -724,12 +730,15 @@
         can be in this code at the same time.
 
         If there's ever more than one build manager running at once, then
-        this code will need some sort of mutex.
+        this code will need some sort of mutex, or run in a single
+        transaction.
         """
         candidate = self._findBuildCandidate()
         if candidate is not None:
-            candidate.markAsBuilding(self)
             transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                candidate.markAsBuilding(self)
+                transaction.commit()
         return candidate
 
     def _findBuildCandidate(self):
@@ -792,13 +801,17 @@
         store = getUtility(IStoreSelector).get(MAIN_STORE, DEFAULT_FLAVOR)
         candidate_jobs = store.execute(query).get_all()
 
-        for (candidate_id,) in candidate_jobs:
-            candidate = getUtility(IBuildQueueSet).get(candidate_id)
-            job_class = job_classes[candidate.job_type]
-            candidate_approved = job_class.postprocessCandidate(
-                candidate, logger)
-            if candidate_approved:
-                return candidate
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            for (candidate_id,) in candidate_jobs:
+                candidate = getUtility(IBuildQueueSet).get(candidate_id)
+                job_class = job_classes[candidate.job_type]
+                candidate_approved = job_class.postprocessCandidate(
+                    candidate, logger)
+                if candidate_approved:
+                    transaction.commit()
+                    return candidate
+            transaction.commit()
 
         return None
 

=== modified file 'lib/lp/buildmaster/model/buildfarmjobbehavior.py'
--- lib/lp/buildmaster/model/buildfarmjobbehavior.py	2011-12-30 02:24:09 +0000
+++ lib/lp/buildmaster/model/buildfarmjobbehavior.py	2012-01-03 12:11:57 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009-2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 # pylint: disable-msg=E0211,E0213
@@ -16,6 +16,7 @@
 import socket
 import xmlrpclib
 
+import transaction
 from twisted.internet import defer
 from zope.component import getUtility
 from zope.interface import implements
@@ -30,6 +31,7 @@
     IBuildFarmJobBehavior,
     )
 from lp.services import encoding
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
 from lp.services.job.interfaces.job import JobStatus
 from lp.services.librarian.interfaces.client import ILibrarianClient
 
@@ -69,6 +71,25 @@
         if slave_build_cookie != expected_cookie:
             raise CorruptBuildCookie("Invalid slave build cookie.")
 
+    def _getBuilderStatusHandler(self, status_text, logger):
+        """Look up the handler method for a given builder status.
+
+        If status is not a known one, logs an error and returns None.
+        """
+        builder_status_handlers = {
+            'BuilderStatus.IDLE': self.updateBuild_IDLE,
+            'BuilderStatus.BUILDING': self.updateBuild_BUILDING,
+            'BuilderStatus.ABORTING': self.updateBuild_ABORTING,
+            'BuilderStatus.ABORTED': self.updateBuild_ABORTED,
+            'BuilderStatus.WAITING': self.updateBuild_WAITING,
+            }
+        handler = builder_status_handlers.get(status_text)
+        if handler is None:
+            logger.critical(
+                "Builder on %s returned unknown status %s; failing it.",
+                self._builder.url, status_text)
+        return handler
+
     def updateBuild(self, queueItem):
         """See `IBuildFarmJobBehavior`."""
         logger = logging.getLogger('slave-scanner')
@@ -76,6 +97,7 @@
         d = self._builder.slaveStatus()
 
         def got_failure(failure):
+            transaction.abort()
             failure.trap(xmlrpclib.Fault, socket.error)
             info = failure.value
             info = ("Could not contact the builder %s, caught a (%s)"
@@ -83,27 +105,22 @@
             raise BuildSlaveFailure(info)
 
         def got_status(slave_status):
-            builder_status_handlers = {
-                'BuilderStatus.IDLE': self.updateBuild_IDLE,
-                'BuilderStatus.BUILDING': self.updateBuild_BUILDING,
-                'BuilderStatus.ABORTING': self.updateBuild_ABORTING,
-                'BuilderStatus.ABORTED': self.updateBuild_ABORTED,
-                'BuilderStatus.WAITING': self.updateBuild_WAITING,
-                }
-
             builder_status = slave_status['builder_status']
-            if builder_status not in builder_status_handlers:
-                logger.critical(
-                    "Builder on %s returned unknown status %s, failing it"
-                    % (self._builder.url, builder_status))
-                self._builder.failBuilder(
+            status_handler = self._getBuilderStatusHandler(
+                builder_status, logger)
+            if status_handler is None:
+                error = (
                     "Unknown status code (%s) returned from status() probe."
                     % builder_status)
-                # XXX: This will leave the build and job in a bad state, but
-                # should never be possible, since our builder statuses are
-                # known.
-                queueItem._builder = None
-                queueItem.setDateStarted(None)
+                transaction.commit()
+                with DatabaseTransactionPolicy(read_only=False):
+                    self._builder.failBuilder(error)
+                    # XXX: This will leave the build and job in a bad
+                    # state, but should never be possible since our
+                    # builder statuses are known.
+                    queueItem._builder = None
+                    queueItem.setDateStarted(None)
+                    transaction.commit()
                 return
 
             # Since logtail is a xmlrpclib.Binary container and it is
@@ -113,9 +130,8 @@
             # will simply remove the proxy.
             logtail = removeSecurityProxy(slave_status.get('logtail'))
 
-            method = builder_status_handlers[builder_status]
             return defer.maybeDeferred(
-                method, queueItem, slave_status, logtail, logger)
+                status_handler, queueItem, slave_status, logtail, logger)
 
         d.addErrback(got_failure)
         d.addCallback(got_status)
@@ -127,22 +143,32 @@
         Log this and reset the record.
         """
         logger.warn(
-            "Builder %s forgot about buildqueue %d -- resetting buildqueue "
-            "record" % (queueItem.builder.url, queueItem.id))
-        queueItem.reset()
+            "Builder %s forgot about buildqueue %d -- "
+            "resetting buildqueue record.",
+            queueItem.builder.url, queueItem.id)
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            queueItem.reset()
+            transaction.commit()
 
     def updateBuild_BUILDING(self, queueItem, slave_status, logtail, logger):
         """Build still building, collect the logtail"""
-        if queueItem.job.status != JobStatus.RUNNING:
-            queueItem.job.start()
-        queueItem.logtail = encoding.guess(str(logtail))
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            if queueItem.job.status != JobStatus.RUNNING:
+                queueItem.job.start()
+            queueItem.logtail = encoding.guess(str(logtail))
+            transaction.commit()
 
     def updateBuild_ABORTING(self, queueItem, slave_status, logtail, logger):
         """Build was ABORTED.
 
         Master-side should wait until the slave finish the process correctly.
         """
-        queueItem.logtail = "Waiting for slave process to be terminated"
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            queueItem.logtail = "Waiting for slave process to be terminated"
+            transaction.commit()
 
     def updateBuild_ABORTED(self, queueItem, slave_status, logtail, logger):
         """ABORTING process has successfully terminated.
@@ -150,11 +176,16 @@
         Clean the builder for another jobs.
         """
         d = queueItem.builder.cleanSlave()
+
         def got_cleaned(ignored):
-            queueItem.builder = None
-            if queueItem.job.status != JobStatus.FAILED:
-                queueItem.job.fail()
-            queueItem.specific_job.jobAborted()
+            transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                queueItem.builder = None
+                if queueItem.job.status != JobStatus.FAILED:
+                    queueItem.job.fail()
+                queueItem.specific_job.jobAborted()
+                transaction.commit()
+
         return d.addCallback(got_cleaned)
 
     def extractBuildStatus(self, slave_status):

=== modified file 'lib/lp/buildmaster/model/packagebuild.py'
--- lib/lp/buildmaster/model/packagebuild.py	2011-12-30 06:14:56 +0000
+++ lib/lp/buildmaster/model/packagebuild.py	2012-01-03 12:17:29 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 __metaclass__ = type
@@ -24,6 +24,7 @@
     Storm,
     Unicode,
     )
+import transaction
 from zope.component import getUtility
 from zope.interface import (
     classProvides,
@@ -50,6 +51,7 @@
 from lp.services.config import config
 from lp.services.database.enumcol import DBEnum
 from lp.services.database.lpstorm import IMasterStore
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
 from lp.services.helpers import filenameToContentType
 from lp.services.librarian.browser import ProxiedLibraryFileAlias
 from lp.services.librarian.interfaces import ILibraryFileAliasSet
@@ -177,19 +179,24 @@
     def storeBuildInfo(build, librarian, slave_status):
         """See `IPackageBuild`."""
         def got_log(lfa_id):
+            dependencies = slave_status.get('dependencies')
+            if dependencies is not None:
+                dependencies = unicode(dependencies)
+
             # log, builder and date_finished are read-only, so we must
             # currently remove the security proxy to set them.
             naked_build = removeSecurityProxy(build)
-            naked_build.log = lfa_id
-            naked_build.builder = build.buildqueue_record.builder
-            # XXX cprov 20060615 bug=120584: Currently buildduration includes
-            # the scanner latency, it should really be asking the slave for
-            # the duration spent building locally.
-            naked_build.date_finished = datetime.datetime.now(pytz.UTC)
-            if slave_status.get('dependencies') is not None:
-                build.dependencies = unicode(slave_status.get('dependencies'))
-            else:
-                build.dependencies = None
+
+            transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                naked_build.log = lfa_id
+                naked_build.builder = build.buildqueue_record.builder
+                # XXX cprov 20060615 bug=120584: Currently buildduration
+                # includes the scanner latency.  It should really be asking
+                # the slave for the duration spent building locally.
+                naked_build.date_finished = datetime.datetime.now(pytz.UTC)
+                build.dependencies = dependencies
+                transaction.commit()
 
         d = build.getLogFromSlave(build)
         return d.addCallback(got_log)
@@ -286,26 +293,46 @@
             job=job, processor=processor,
             virtualized=specific_job.virtualized)
         Store.of(self).add(queue_entry)
+
         return queue_entry
 
     def handleStatus(self, status, librarian, slave_status):
         """See `IPackageBuild`."""
+        # Avoid circular imports.
         from lp.buildmaster.manager import BUILDD_MANAGER_LOG_NAME
+
         logger = logging.getLogger(BUILDD_MANAGER_LOG_NAME)
         send_notification = status in self.ALLOWED_STATUS_NOTIFICATIONS
         method = getattr(self, '_handleStatus_' + status, None)
         if method is None:
-            logger.critical("Unknown BuildStatus '%s' for builder '%s'"
-                            % (status, self.buildqueue_record.builder.url))
-            return
+            logger.critical(
+                "Unknown BuildStatus '%s' for builder '%s'",
+                status, self.buildqueue_record.builder.url)
+            return None
+
         d = method(librarian, slave_status, logger, send_notification)
         return d
 
+    def _destroy_buildqueue_record(self, unused_arg):
+        """Destroy this build's `BuildQueue` record."""
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            self.buildqueue_record.destroySelf()
+            transaction.commit()
+
     def _release_builder_and_remove_queue_item(self):
         # Release the builder for another job.
         d = self.buildqueue_record.builder.cleanSlave()
         # Remove BuildQueue record.
-        return d.addCallback(lambda x: self.buildqueue_record.destroySelf())
+        return d.addCallback(self._destroy_buildqueue_record)
+
+    def _notify_if_appropriate(self, appropriate=True, extra_info=None):
+        """If `appropriate`, call `self.notify` in a write transaction."""
+        if appropriate:
+            transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                self.notify(extra_info=extra_info)
+                transaction.commit()
 
     def _handleStatus_OK(self, librarian, slave_status, logger,
                          send_notification):
@@ -321,16 +348,19 @@
             self.buildqueue_record.specific_job.build.title,
             self.buildqueue_record.builder.name))
 
-        # If this is a binary package build, discard it if its source is
-        # no longer published.
+        # If this is a binary package build for a source that is no
+        # longer published, discard it.
         if self.build_farm_job_type == BuildFarmJobType.PACKAGEBUILD:
             build = self.buildqueue_record.specific_job.build
             if not build.current_source_publication:
-                build.status = BuildStatus.SUPERSEDED
+                transaction.commit()
+                with DatabaseTransactionPolicy(read_only=False):
+                    build.status = BuildStatus.SUPERSEDED
+                    transaction.commit()
                 return self._release_builder_and_remove_queue_item()
 
-        # Explode before collect a binary that is denied in this
-        # distroseries/pocket
+        # Explode rather than collect a binary that is denied in this
+        # distroseries/pocket.
         if not self.archive.allowUpdatesToReleasePocket():
             assert self.distro_series.canUploadToPocket(self.pocket), (
                 "%s (%s) can not be built for pocket %s: illegal status"
@@ -375,18 +405,26 @@
             # files from the slave.
             if successful_copy_from_slave:
                 logger.info(
-                    "Gathered %s %d completely. Moving %s to uploader queue."
-                    % (self.__class__.__name__, self.id, upload_leaf))
+                    "Gathered %s %d completely. "
+                    "Moving %s to uploader queue.",
+                    self.__class__.__name__, self.id, upload_leaf)
                 target_dir = os.path.join(root, "incoming")
-                self.status = BuildStatus.UPLOADING
+                resulting_status = BuildStatus.UPLOADING
             else:
                 logger.warning(
-                    "Copy from slave for build %s was unsuccessful.", self.id)
-                self.status = BuildStatus.FAILEDTOUPLOAD
-                if send_notification:
-                    self.notify(
-                        extra_info='Copy from slave was unsuccessful.')
+                    "Copy from slave for build %s was unsuccessful.",
+                    self.id)
                 target_dir = os.path.join(root, "failed")
+                resulting_status = BuildStatus.FAILEDTOUPLOAD
+
+            transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                self.status = resulting_status
+                transaction.commit()
+
+            if not successful_copy_from_slave:
+                self._notify_if_appropriate(
+                    send_notification, "Copy from slave was unsuccessful.")
 
             if not os.path.exists(target_dir):
                 os.mkdir(target_dir)
@@ -394,10 +432,6 @@
             # Release the builder for another job.
             d = self._release_builder_and_remove_queue_item()
 
-            # Commit so there are no race conditions with archiveuploader
-            # about self.status.
-            Store.of(self).commit()
-
             # Move the directory used to grab the binaries into
             # the incoming directory so the upload processor never
             # sees half-finished uploads.
@@ -421,14 +455,15 @@
         set the job status as FAILEDTOBUILD, store available info and
         remove Buildqueue entry.
         """
-        self.status = BuildStatus.FAILEDTOBUILD
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            self.status = BuildStatus.FAILEDTOBUILD
+            transaction.commit()
 
         def build_info_stored(ignored):
-            if send_notification:
-                self.notify()
+            self._notify_if_appropriate(send_notification)
             d = self.buildqueue_record.builder.cleanSlave()
-            return d.addCallback(
-                lambda x: self.buildqueue_record.destroySelf())
+            return d.addCallback(self._destroy_buildqueue_record)
 
         d = self.storeBuildInfo(self, librarian, slave_status)
         return d.addCallback(build_info_stored)
@@ -441,16 +476,16 @@
         MANUALDEPWAIT, store available information, remove BuildQueue
         entry and release builder slave for another job.
         """
-        self.status = BuildStatus.MANUALDEPWAIT
+        with DatabaseTransactionPolicy(read_only=False):
+            self.status = BuildStatus.MANUALDEPWAIT
+            transaction.commit()
 
         def build_info_stored(ignored):
             logger.critical("***** %s is MANUALDEPWAIT *****"
                             % self.buildqueue_record.builder.name)
-            if send_notification:
-                self.notify()
+            self._notify_if_appropriate(send_notification)
             d = self.buildqueue_record.builder.cleanSlave()
-            return d.addCallback(
-                lambda x: self.buildqueue_record.destroySelf())
+            return d.addCallback(self._destroy_buildqueue_record)
 
         d = self.storeBuildInfo(self, librarian, slave_status)
         return d.addCallback(build_info_stored)
@@ -463,20 +498,29 @@
         job as CHROOTFAIL, store available information, remove BuildQueue
         and release the builder.
         """
-        self.status = BuildStatus.CHROOTWAIT
+        with DatabaseTransactionPolicy(read_only=False):
+            self.status = BuildStatus.CHROOTWAIT
+            transaction.commit()
 
         def build_info_stored(ignored):
-            logger.critical("***** %s is CHROOTWAIT *****" %
-                            self.buildqueue_record.builder.name)
-            if send_notification:
-                self.notify()
+            logger.critical(
+                "***** %s is CHROOTWAIT *****",
+                self.buildqueue_record.builder.name)
+
+            self._notify_if_appropriate(send_notification)
             d = self.buildqueue_record.builder.cleanSlave()
-            return d.addCallback(
-                lambda x: self.buildqueue_record.destroySelf())
+            return d.addCallback(self._destroy_buildqueue_record)
 
         d = self.storeBuildInfo(self, librarian, slave_status)
         return d.addCallback(build_info_stored)
 
+    def _reset_buildqueue_record(self, ignored_arg=None):
+        """Reset the `BuildQueue` record, in a write transaction."""
+        transaction.commit()
+        with DatabaseTransactionPolicy(read_only=False):
+            self.buildqueue_record.reset()
+            transaction.commit()
+
     def _handleStatus_BUILDERFAIL(self, librarian, slave_status, logger,
                                   send_notification):
         """Handle builder failures.
@@ -490,11 +534,8 @@
         self.buildqueue_record.builder.failBuilder(
             "Builder returned BUILDERFAIL when asked for its status")
 
-        def build_info_stored(ignored):
-            # simply reset job
-            self.buildqueue_record.reset()
         d = self.storeBuildInfo(self, librarian, slave_status)
-        return d.addCallback(build_info_stored)
+        return d.addCallback(self._reset_buildqueue_record)
 
     def _handleStatus_GIVENBACK(self, librarian, slave_status, logger,
                                 send_notification):
@@ -514,7 +555,7 @@
             # the next Paris Summit, infinity has some ideas about how
             # to use this content. For now we just ensure it's stored.
             d = self.buildqueue_record.builder.cleanSlave()
-            self.buildqueue_record.reset()
+            self._reset_buildqueue_record()
             return d
 
         d = self.storeBuildInfo(self, librarian, slave_status)

=== added file 'lib/lp/buildmaster/testing.py'
--- lib/lp/buildmaster/testing.py	1970-01-01 00:00:00 +0000
+++ lib/lp/buildmaster/testing.py	2011-12-20 21:16:35 +0000
@@ -0,0 +1,80 @@
+# Copyright 2011 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""Testing helpers for buildmaster code."""
+
+__metaclass__ = type
+__all__ = [
+    "BuilddManagerTestFixture",
+    ]
+
+from contextlib import contextmanager
+from functools import wraps
+
+import fixtures
+import transaction
+
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
+
+
+class BuilddManagerTestFixture(fixtures.Fixture):
+    """Helps provide an environment more like `BuilddManager` provides.
+
+    This mimics the default transaction access policy of `BuilddManager`,
+    though it can be configured with a different policy by passing in `store`
+    and/or `read_only`. See `BuilddManager.enterReadOnlyDatabasePolicy`.
+
+    Because this will shift into a read-only database transaction access mode,
+    individual tests that need to do more setup can use the `extraSetUp()`
+    context manager to temporarily shift back to a read-write mode.
+    """
+
+    def __init__(self, store=None, read_only=True):
+        super(BuilddManagerTestFixture, self).__init__()
+        self.policy = DatabaseTransactionPolicy(
+            store=store, read_only=read_only)
+
+    def setUp(self):
+        # Commit everything done so far then shift into a read-only
+        # transaction access mode by default.
+        super(BuilddManagerTestFixture, self).setUp()
+        transaction.commit()
+        self.policy.__enter__()
+        self.addCleanup(self.policy.__exit__, None, None, None)
+
+    @staticmethod
+    def extraSetUp(func=None):
+        """Temporarily enter a read-write transaction to do extra setup.
+
+        For example:
+
+          with BuilddManagerTestFixture.extraSetUp():
+              removeSecurityProxy(self.build).date_finished = None
+
+        On exit it will commit the changes and restore the previous
+        transaction access mode.
+
+        Alternatively it can be used as a decorator:
+
+          @BuilddManagerTestFixture.extraSetUp
+          def makeSomethingOrOther(self):
+              return ...
+
+        Like with the context manager, on return it will commit the changes
+        and restore the previous transaction access mode.
+        """
+        if func is None:
+            @contextmanager
+            def context():
+                with DatabaseTransactionPolicy(read_only=False):
+                    yield
+                    transaction.commit()
+            return context()
+        else:
+            @wraps(func)
+            def wrapper(*args, **kwargs):
+                with DatabaseTransactionPolicy(read_only=False):
+                    result = func(*args, **kwargs)
+                    transaction.commit()
+                    return result
+            return wrapper

=== modified file 'lib/lp/buildmaster/tests/test_builder.py'
--- lib/lp/buildmaster/tests/test_builder.py	2011-12-30 06:14:56 +0000
+++ lib/lp/buildmaster/tests/test_builder.py	2012-01-03 13:44:35 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009-2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Test Builder features."""
@@ -15,6 +15,7 @@
     AsynchronousDeferredRunTestForBrokenTwisted,
     SynchronousDeferredRunTest,
     )
+import transaction
 from twisted.internet.defer import (
     CancelledError,
     DeferredList,
@@ -45,6 +46,7 @@
     )
 from lp.buildmaster.model.buildfarmjobbehavior import IdleBuildBehavior
 from lp.buildmaster.model.buildqueue import BuildQueue
+from lp.buildmaster.testing import BuilddManagerTestFixture
 from lp.buildmaster.tests.mock_slaves import (
     AbortedSlave,
     AbortingSlave,
@@ -60,6 +62,7 @@
     TrivialBehavior,
     WaitingSlave,
     )
+from lp.registry.interfaces.pocket import PackagePublishingPocket
 from lp.services.config import config
 from lp.services.database.sqlbase import flush_database_updates
 from lp.services.job.interfaces.job import JobStatus
@@ -82,55 +85,69 @@
     TestCaseWithFactory,
     )
 from lp.testing.fakemethod import FakeMethod
-from lp.testing.layers import (
-    DatabaseFunctionalLayer,
-    LaunchpadZopelessLayer,
-    )
+from lp.testing.layers import LaunchpadZopelessLayer
 
 
 class TestBuilderBasics(TestCaseWithFactory):
     """Basic unit tests for `Builder`."""
 
-    layer = DatabaseFunctionalLayer
+    layer = LaunchpadZopelessLayer
+
+    def setUp(self):
+        super(TestBuilderBasics, self).setUp()
+        self.useFixture(BuilddManagerTestFixture())
 
     def test_providesInterface(self):
         # Builder provides IBuilder
-        builder = self.factory.makeBuilder()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder()
         self.assertProvides(builder, IBuilder)
 
     def test_default_values(self):
-        builder = self.factory.makeBuilder()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder()
         # Make sure the Storm cache gets the values that the database
         # initializes.
         flush_database_updates()
+        self.useFixture(BuilddManagerTestFixture())
         self.assertEqual(0, builder.failure_count)
 
     def test_getCurrentBuildFarmJob(self):
-        bq = self.factory.makeSourcePackageRecipeBuildJob(3333)
-        builder = self.factory.makeBuilder()
-        bq.markAsBuilding(builder)
+        with BuilddManagerTestFixture.extraSetUp():
+            bq = self.factory.makeSourcePackageRecipeBuildJob(3333)
+            builder = self.factory.makeBuilder()
+            bq.markAsBuilding(builder)
+        self.useFixture(BuilddManagerTestFixture())
         self.assertEqual(
             bq, builder.getCurrentBuildFarmJob().buildqueue_record)
 
     def test_getBuildQueue(self):
-        buildqueueset = getUtility(IBuildQueueSet)
-        active_jobs = buildqueueset.getActiveBuildJobs()
-        [active_job] = active_jobs
-        builder = active_job.builder
+        with BuilddManagerTestFixture.extraSetUp():
+            buildqueueset = getUtility(IBuildQueueSet)
+            active_jobs = buildqueueset.getActiveBuildJobs()
+            [active_job] = active_jobs
+            builder = active_job.builder
 
         bq = builder.getBuildQueue()
         self.assertEqual(active_job, bq)
 
-        active_job.builder = None
+        with BuilddManagerTestFixture.extraSetUp():
+            active_job.builder = None
+
         bq = builder.getBuildQueue()
         self.assertIs(None, bq)
 
     def test_setting_builderok_resets_failure_count(self):
-        builder = removeSecurityProxy(self.factory.makeBuilder())
-        builder.failure_count = 1
-        builder.builderok = False
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = removeSecurityProxy(self.factory.makeBuilder())
+            builder.failure_count = 1
+            builder.builderok = False
+
         self.assertEqual(1, builder.failure_count)
-        builder.builderok = True
+
+        with BuilddManagerTestFixture.extraSetUp():
+            builder.builderok = True
+
         self.assertEqual(0, builder.failure_count)
 
 
@@ -142,6 +159,7 @@
     def setUp(self):
         super(TestBuilder, self).setUp()
         self.slave_helper = self.useFixture(SlaveTestHelpers())
+        self.useFixture(BuilddManagerTestFixture())
 
     def test_updateStatus_aborts_lost_and_broken_slave(self):
         # A slave that's 'lost' should be aborted; when the slave is
@@ -152,18 +170,20 @@
         d = lostbuilding_builder.updateStatus(BufferLogger())
         def check_slave_status(failure):
             self.assertIn('abort', slave.call_log)
-            # 'Fault' comes from the LostBuildingBrokenSlave, this is
+            # 'Fault' comes from the LostBuildingBrokenSlave.  This is
             # just testing that the value is passed through.
             self.assertIsInstance(failure.value, xmlrpclib.Fault)
         return d.addBoth(check_slave_status)
 
     def test_resumeSlaveHost_nonvirtual(self):
-        builder = self.factory.makeBuilder(virtualized=False)
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder(virtualized=False)
         d = builder.resumeSlaveHost()
         return assert_fails_with(d, CannotResumeHost)
 
     def test_resumeSlaveHost_no_vmhost(self):
-        builder = self.factory.makeBuilder(virtualized=True, vm_host=None)
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder(virtualized=True, vm_host=None)
         d = builder.resumeSlaveHost()
         return assert_fails_with(d, CannotResumeHost)
 
@@ -174,7 +194,9 @@
         config.push('reset', reset_config)
         self.addCleanup(config.pop, 'reset')
 
-        builder = self.factory.makeBuilder(virtualized=True, vm_host="pop")
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder(
+                virtualized=True, vm_host="pop")
         d = builder.resumeSlaveHost()
         def got_resume(output):
             self.assertEqual(('parp', ''), output)
@@ -186,7 +208,9 @@
             vm_resume_command: /bin/false"""
         config.push('reset fail', reset_fail_config)
         self.addCleanup(config.pop, 'reset fail')
-        builder = self.factory.makeBuilder(virtualized=True, vm_host="pop")
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder(
+                virtualized=True, vm_host="pop")
         d = builder.resumeSlaveHost()
         return assert_fails_with(d, CannotResumeHost)
 
@@ -196,11 +220,14 @@
             vm_resume_command: /bin/false"""
         config.push('reset fail', reset_fail_config)
         self.addCleanup(config.pop, 'reset fail')
-        builder = self.factory.makeBuilder(virtualized=True, vm_host="pop")
-        builder.builderok = True
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder(
+                virtualized=True, vm_host="pop")
+            builder.builderok = True
         d = builder.handleTimeout(BufferLogger(), 'blah')
         return assert_fails_with(d, CannotResumeHost)
 
+    @BuilddManagerTestFixture.extraSetUp
     def _setupBuilder(self):
         processor = self.factory.makeProcessor(name="i386")
         builder = self.factory.makeBuilder(
@@ -215,6 +242,7 @@
         distroseries.nominatedarchindep = das
         return builder, distroseries, das
 
+    @BuilddManagerTestFixture.extraSetUp
     def _setupRecipeBuildAndBuilder(self):
         # Helper function to make a builder capable of building a
         # recipe, returning both.
@@ -223,6 +251,7 @@
             distroseries=distroseries)
         return builder, build
 
+    @BuilddManagerTestFixture.extraSetUp
     def _setupBinaryBuildAndBuilder(self):
         # Helper function to make a builder capable of building a
         # binary package, returning both.
@@ -235,7 +264,8 @@
         # findAndStartJob finds the next queued job using _findBuildCandidate.
         # We don't care about the type of build at all.
         builder, build = self._setupRecipeBuildAndBuilder()
-        candidate = build.queueBuild()
+        with BuilddManagerTestFixture.extraSetUp():
+            candidate = build.queueBuild()
         # _findBuildCandidate is tested elsewhere, we just make sure that
         # findAndStartJob delegates to it.
         removeSecurityProxy(builder)._findBuildCandidate = FakeMethod(
@@ -248,7 +278,8 @@
         # and then starts it.
         # We don't care about the type of build at all.
         builder, build = self._setupRecipeBuildAndBuilder()
-        candidate = build.queueBuild()
+        with BuilddManagerTestFixture.extraSetUp():
+            candidate = build.queueBuild()
         removeSecurityProxy(builder)._findBuildCandidate = FakeMethod(
             result=candidate)
         d = builder.findAndStartJob()
@@ -261,7 +292,8 @@
         # We need to send a ping to the builder to work around a bug
         # where sometimes the first network packet sent is dropped.
         builder, build = self._setupBinaryBuildAndBuilder()
-        candidate = build.queueBuild()
+        with BuilddManagerTestFixture.extraSetUp():
+            candidate = build.queueBuild()
         removeSecurityProxy(builder)._findBuildCandidate = FakeMethod(
             result=candidate)
         d = builder.findAndStartJob()
@@ -274,7 +306,8 @@
         # Builder.slave is a BuilderSlave that points at the actual Builder.
         # The Builder is only ever used in scripts that run outside of the
         # security context.
-        builder = removeSecurityProxy(self.factory.makeBuilder())
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = removeSecurityProxy(self.factory.makeBuilder())
         self.assertEqual(builder.url, builder.slave.url)
 
     def test_recovery_of_aborted_virtual_slave(self):
@@ -319,7 +352,8 @@
         # rescueIfLost does not attempt to abort or clean a builder that is
         # WAITING.
         waiting_slave = WaitingSlave()
-        builder = MockBuilder("mock_builder", waiting_slave, TrivialBehavior())
+        builder = MockBuilder(
+            "mock_builder", waiting_slave, TrivialBehavior())
         d = builder.rescueIfLost()
         def check_slave_calls(ignored):
             self.assertNotIn('abort', waiting_slave.call_log)
@@ -333,7 +367,8 @@
         # builder is reset for a new build, and the corrupt build is
         # discarded.
         waiting_slave = WaitingSlave()
-        builder = MockBuilder("mock_builder", waiting_slave, CorruptBehavior())
+        builder = MockBuilder(
+            "mock_builder", waiting_slave, CorruptBehavior())
         d = builder.rescueIfLost()
         def check_slave_calls(ignored):
             self.assertNotIn('abort', waiting_slave.call_log)
@@ -344,7 +379,8 @@
         # rescueIfLost does not attempt to abort or clean a builder that is
         # BUILDING.
         building_slave = BuildingSlave()
-        builder = MockBuilder("mock_builder", building_slave, TrivialBehavior())
+        builder = MockBuilder(
+            "mock_builder", building_slave, TrivialBehavior())
         d = builder.rescueIfLost()
         def check_slave_calls(ignored):
             self.assertNotIn('abort', building_slave.call_log)
@@ -355,7 +391,8 @@
         # If a slave is BUILDING with a build id we don't recognize, then we
         # abort the build, thus stopping it in its tracks.
         building_slave = BuildingSlave()
-        builder = MockBuilder("mock_builder", building_slave, CorruptBehavior())
+        builder = MockBuilder(
+            "mock_builder", building_slave, CorruptBehavior())
         d = builder.rescueIfLost()
         def check_slave_calls(ignored):
             self.assertIn('abort', building_slave.call_log)
@@ -368,10 +405,11 @@
         # too so that we don't traceback when the wrong behaviour tries
         # to access a non-existent job.
         builder, build = self._setupBinaryBuildAndBuilder()
-        candidate = build.queueBuild()
-        building_slave = BuildingSlave()
-        builder.setSlaveForTesting(building_slave)
-        candidate.markAsBuilding(builder)
+        with BuilddManagerTestFixture.extraSetUp():
+            candidate = build.queueBuild()
+            building_slave = BuildingSlave()
+            builder.setSlaveForTesting(building_slave)
+            candidate.markAsBuilding(builder)
 
         # At this point we should see a valid behaviour on the builder:
         self.assertFalse(
@@ -379,8 +417,8 @@
                 builder.current_build_behavior, IdleBuildBehavior))
 
         # Now reset the job and try to rescue the builder.
-        candidate.destroySelf()
-        self.layer.txn.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            candidate.destroySelf()
         builder = getUtility(IBuilderSet)[builder.name]
         d = builder.rescueIfLost()
         def check_builder(ignored):
@@ -400,13 +438,14 @@
     def setUp(self):
         super(TestBuilderSlaveStatus, self).setUp()
         self.slave_helper = self.useFixture(SlaveTestHelpers())
+        self.builder = self.factory.makeBuilder()
+        self.useFixture(BuilddManagerTestFixture())
 
     def assertStatus(self, slave, builder_status=None,
                      build_status=None, logtail=False, filemap=None,
                      dependencies=None):
-        builder = self.factory.makeBuilder()
-        builder.setSlaveForTesting(slave)
-        d = builder.slaveStatus()
+        self.builder.setSlaveForTesting(slave)
+        d = self.builder.slaveStatus()
 
         def got_status(status_dict):
             expected = {}
@@ -451,21 +490,19 @@
 
     def test_isAvailable_with_not_builderok(self):
         # isAvailable() is a wrapper around slaveStatusSentence()
-        builder = self.factory.makeBuilder()
-        builder.builderok = False
-        d = builder.isAvailable()
+        with BuilddManagerTestFixture.extraSetUp():
+            self.builder.builderok = False
+        d = self.builder.isAvailable()
         return d.addCallback(self.assertFalse)
 
     def test_isAvailable_with_slave_fault(self):
-        builder = self.factory.makeBuilder()
-        builder.setSlaveForTesting(BrokenSlave())
-        d = builder.isAvailable()
+        self.builder.setSlaveForTesting(BrokenSlave())
+        d = self.builder.isAvailable()
         return d.addCallback(self.assertFalse)
 
     def test_isAvailable_with_slave_idle(self):
-        builder = self.factory.makeBuilder()
-        builder.setSlaveForTesting(OkSlave())
-        d = builder.isAvailable()
+        self.builder.setSlaveForTesting(OkSlave())
+        d = self.builder.isAvailable()
         return d.addCallback(self.assertTrue)
 
 
@@ -499,6 +536,8 @@
             builder.builderok = True
             builder.manual = False
 
+        self.useFixture(BuilddManagerTestFixture())
+
 
 class TestFindBuildCandidateGeneralCases(TestFindBuildCandidateBase):
     # Test usage of findBuildCandidate not specific to any archive type.
@@ -507,10 +546,11 @@
         # IBuilder._findBuildCandidate identifies if there are builds
         # for superseded source package releases in the queue and marks
         # the corresponding build record as SUPERSEDED.
-        archive = self.factory.makeArchive()
-        self.publisher.getPubSource(
-            sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
-            archive=archive).createMissingBuilds()
+        with BuilddManagerTestFixture.extraSetUp():
+            archive = self.factory.makeArchive()
+            self.publisher.getPubSource(
+                sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
+                archive=archive).createMissingBuilds()
         old_candidate = removeSecurityProxy(
             self.frog_builder)._findBuildCandidate()
 
@@ -521,7 +561,8 @@
 
         # Now supersede the source package:
         publication = build.current_source_publication
-        publication.status = PackagePublishingStatus.SUPERSEDED
+        with BuilddManagerTestFixture.extraSetUp():
+            publication.status = PackagePublishingStatus.SUPERSEDED
 
         # The candidate returned is now a different one:
         new_candidate = removeSecurityProxy(
@@ -531,13 +572,32 @@
         # And the old_candidate is superseded:
         self.assertEqual(BuildStatus.SUPERSEDED, build.status)
 
+    def test_findBuildCandidate_postprocesses_in_read_write_policy(self):
+        # _findBuildCandidate invokes BuildFarmJob.postprocessCandidate,
+        # which may modify the database.  This happens in a read-write
+        # transaction even if _findBuildCandidate itself runs in a
+        # read-only transaction policy.
+
+        # PackageBuildJob.postprocessCandidate will attempt to delete
+        # security builds.
+        with BuilddManagerTestFixture.extraSetUp():
+            pub = self.publisher.getPubSource(
+                sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
+                archive=self.factory.makeArchive(),
+                pocket=PackagePublishingPocket.SECURITY)
+            pub.createMissingBuilds()
+        removeSecurityProxy(self.frog_builder)._findBuildCandidate()
+        # Passes without a "transaction is read-only" error...
+        transaction.commit()
+
     def test_acquireBuildCandidate_marks_building(self):
         # acquireBuildCandidate() should call _findBuildCandidate and
         # mark the build as building.
-        archive = self.factory.makeArchive()
-        self.publisher.getPubSource(
-            sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
-            archive=archive).createMissingBuilds()
+        with BuilddManagerTestFixture.extraSetUp():
+            archive = self.factory.makeArchive()
+            self.publisher.getPubSource(
+                sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
+                archive=archive).createMissingBuilds()
         candidate = removeSecurityProxy(
             self.frog_builder).acquireBuildCandidate()
         self.assertEqual(JobStatus.RUNNING, candidate.job.status)
@@ -567,6 +627,8 @@
             sourcename="gedit", status=PackagePublishingStatus.PUBLISHED,
             archive=self.ppa_joe).createMissingBuilds()
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def test_findBuildCandidate_first_build_started(self):
         # The allocation rule for PPA dispatching doesn't apply when
         # there's only one builder available.
@@ -579,8 +641,9 @@
 
         # If bob is in a failed state the joesppa build is still
         # returned.
-        self.bob_builder.builderok = False
-        self.bob_builder.manual = False
+        with BuilddManagerTestFixture.extraSetUp():
+            self.bob_builder.builderok = False
+            self.bob_builder.manual = False
         next_job = removeSecurityProxy(
             self.frog_builder)._findBuildCandidate()
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(next_job)
@@ -592,6 +655,7 @@
     ppa_joe_private = False
     ppa_jim_private = False
 
+    @BuilddManagerTestFixture.extraSetUp
     def _setBuildsBuildingForArch(self, builds_list, num_builds,
                                   archtag="i386"):
         """Helper function.
@@ -609,7 +673,10 @@
     def setUp(self):
         """Publish some builds for the test archive."""
         super(TestFindBuildCandidatePPABase, self).setUp()
+        self.extraSetUp()
 
+    @BuilddManagerTestFixture.extraSetUp
+    def extraSetUp(self):
         # Create two PPAs and add some builds to each.
         self.ppa_joe = self.factory.makeArchive(
             name="joesppa", private=self.ppa_joe_private)
@@ -670,7 +737,8 @@
     def test_findBuildCandidate_first_build_finished(self):
         # When joe's first ppa build finishes, his fourth i386 build
         # will be the next build candidate.
-        self.joe_builds[0].status = BuildStatus.FAILEDTOBUILD
+        with BuilddManagerTestFixture.extraSetUp():
+            self.joe_builds[0].status = BuildStatus.FAILEDTOBUILD
         next_job = removeSecurityProxy(self.builder4)._findBuildCandidate()
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(next_job)
         self.failUnlessEqual('joesppa', build.archive.name)
@@ -678,10 +746,12 @@
     def test_findBuildCandidate_with_disabled_archive(self):
         # Disabled archives should not be considered for dispatching
         # builds.
-        disabled_job = removeSecurityProxy(self.builder4)._findBuildCandidate()
-        build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(
-            disabled_job)
-        build.archive.disable()
+        disabled_job = removeSecurityProxy(
+            self.builder4)._findBuildCandidate()
+        with BuilddManagerTestFixture.extraSetUp():
+            build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(
+                disabled_job)
+            build.archive.disable()
         next_job = removeSecurityProxy(self.builder4)._findBuildCandidate()
         self.assertNotEqual(disabled_job, next_job)
 
@@ -701,7 +771,8 @@
         # dispatched because the builder has to fetch the source files
         # from the (password protected) repo area, not the librarian.
         pub = build.current_source_publication
-        pub.status = PackagePublishingStatus.PENDING
+        with BuilddManagerTestFixture.extraSetUp():
+            pub.status = PackagePublishingStatus.PENDING
         candidate = removeSecurityProxy(self.builder4)._findBuildCandidate()
         self.assertNotEqual(next_job.id, candidate.id)
 
@@ -711,6 +782,10 @@
     def setUp(self):
         """Publish some builds for the test archive."""
         super(TestFindBuildCandidateDistroArchive, self).setUp()
+        self.extraSetUp()
+
+    @BuilddManagerTestFixture.extraSetUp
+    def extraSetUp(self):
         # Create a primary archive and publish some builds for the
         # queue.
         self.non_ppa = self.factory.makeArchive(
@@ -726,7 +801,6 @@
     def test_findBuildCandidate_for_non_ppa(self):
         # Normal archives are not restricted to serial builds per
         # arch.
-
         next_job = removeSecurityProxy(
             self.frog_builder)._findBuildCandidate()
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(next_job)
@@ -735,8 +809,9 @@
 
         # Now even if we set the build building, we'll still get the
         # second non-ppa build for the same archive as the next candidate.
-        build.status = BuildStatus.BUILDING
-        build.builder = self.frog_builder
+        with BuilddManagerTestFixture.extraSetUp():
+            build.status = BuildStatus.BUILDING
+            build.builder = self.frog_builder
         next_job = removeSecurityProxy(
             self.frog_builder)._findBuildCandidate()
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(next_job)
@@ -753,7 +828,9 @@
         self.assertEqual(self.gedit_build.buildqueue_record.lastscore, 2505)
         self.assertEqual(self.firefox_build.buildqueue_record.lastscore, 2505)
 
-        recipe_build_job = self.factory.makeSourcePackageRecipeBuildJob(9999)
+        with BuilddManagerTestFixture.extraSetUp():
+            recipe_build_job = (
+                self.factory.makeSourcePackageRecipeBuildJob(9999))
 
         self.assertEqual(recipe_build_job.lastscore, 9999)
 
@@ -767,6 +844,7 @@
     # These tests operate in a "recipe builds only" setting.
     # Please see also bug #507782.
 
+    @BuilddManagerTestFixture.extraSetUp
     def clearBuildQueue(self):
         """Delete all `BuildQueue`, XXXJOb and `Job` instances."""
         store = getUtility(IStoreSelector).get(MAIN_STORE, DEFAULT_FLAVOR)
@@ -776,6 +854,10 @@
     def setUp(self):
         """Publish some builds for the test archive."""
         super(TestFindRecipeBuildCandidates, self).setUp()
+        self.extraSetUp()
+
+    @BuilddManagerTestFixture.extraSetUp
+    def extraSetUp(self):
         # Create a primary archive and publish some builds for the
         # queue.
         self.non_ppa = self.factory.makeArchive(
@@ -821,6 +903,8 @@
 
         self.buildfarmjob = self.build.buildqueue_record.specific_job
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def test_idle_behavior_when_no_current_build(self):
         """We return an idle behavior when there is no behavior specified
         nor a current build.
@@ -841,7 +925,8 @@
         """The current behavior is set automatically from the current job."""
         # Set the builder attribute on the buildqueue record so that our
         # builder will think it has a current build.
-        self.build.buildqueue_record.builder = self.builder
+        with BuilddManagerTestFixture.extraSetUp():
+            self.build.buildqueue_record.builder = self.builder
 
         self.assertIsInstance(
             self.builder.current_build_behavior, BinaryPackageBuildBehavior)
@@ -1134,15 +1219,17 @@
         super(TestSlaveWithLibrarian, self).setUp()
         self.slave_helper = self.useFixture(SlaveTestHelpers())
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def test_ensurepresent_librarian(self):
         # ensurepresent, when given an http URL for a file will download the
         # file from that URL and report that the file is present, and it was
         # downloaded.
 
         # Use the Librarian because it's a "convenient" web server.
-        lf = self.factory.makeLibraryFileAlias(
-            'HelloWorld.txt', content="Hello World")
-        self.layer.txn.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            lf = self.factory.makeLibraryFileAlias(
+                'HelloWorld.txt', content="Hello World")
         self.slave_helper.getServerSlave()
         slave = self.slave_helper.getClientSlave()
         d = slave.ensurepresent(
@@ -1155,9 +1242,9 @@
         # filename made from the sha1 of the content underneath the
         # 'filecache' directory.
         content = "Hello World"
-        lf = self.factory.makeLibraryFileAlias(
-            'HelloWorld.txt', content=content)
-        self.layer.txn.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            lf = self.factory.makeLibraryFileAlias(
+                'HelloWorld.txt', content=content)
         expected_url = '%s/filecache/%s' % (
             self.slave_helper.BASE_URL, lf.content.sha1)
         self.slave_helper.getServerSlave()
@@ -1183,7 +1270,6 @@
         def got_files(ignored):
             # Called back when getFiles finishes.  Make sure all the
             # content is as expected.
-            got_contents = []
             for sha1 in filemap:
                 local_file = filemap[sha1]
                 file = open(local_file)
@@ -1199,11 +1285,12 @@
         dl = []
         for content in contents:
             filename = content + '.txt'
-            lf = self.factory.makeLibraryFileAlias(filename, content=content)
+            with BuilddManagerTestFixture.extraSetUp():
+                lf = self.factory.makeLibraryFileAlias(
+                    filename, content=content)
             content_map[lf.content.sha1] = content
             fd, filemap[lf.content.sha1] = tempfile.mkstemp()
             self.addCleanup(os.remove, filemap[lf.content.sha1])
-            self.layer.txn.commit()
             d = slave.ensurepresent(lf.content.sha1, lf.http_url, "", "")
             dl.append(d)
 

=== modified file 'lib/lp/buildmaster/tests/test_manager.py'
--- lib/lp/buildmaster/tests/test_manager.py	2012-01-01 02:58:52 +0000
+++ lib/lp/buildmaster/tests/test_manager.py	2012-01-03 13:44:35 +0000
@@ -1,8 +1,9 @@
-# Copyright 2009-2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Tests for the renovated slave scanner aka BuilddManager."""
 
+from collections import namedtuple
 import os
 import signal
 import time
@@ -13,7 +14,6 @@
     assert_fails_with,
     AsynchronousDeferredRunTest,
     )
-import transaction
 from twisted.internet import (
     defer,
     reactor,
@@ -34,15 +34,19 @@
     SlaveScanner,
     )
 from lp.buildmaster.model.builder import Builder
+from lp.buildmaster.model.packagebuild import PackageBuild
+from lp.buildmaster.testing import BuilddManagerTestFixture
 from lp.buildmaster.tests.harness import BuilddManagerTestSetup
 from lp.buildmaster.tests.mock_slaves import (
     BrokenSlave,
     BuildingSlave,
     make_publisher,
     OkSlave,
+    WaitingSlave,
     )
 from lp.registry.interfaces.distribution import IDistributionSet
 from lp.services.config import config
+from lp.services.database.constants import UTC_NOW
 from lp.services.log.logger import BufferLogger
 from lp.soyuz.interfaces.binarypackagebuild import IBinaryPackageBuildSet
 from lp.testing import (
@@ -58,10 +62,13 @@
     LaunchpadZopelessLayer,
     ZopelessDatabaseLayer,
     )
-from lp.testing.sampledata import BOB_THE_BUILDER_NAME
-
-
-class TestSlaveScannerScan(TestCase):
+from lp.testing.sampledata import (
+    BOB_THE_BUILDER_NAME,
+    FROG_THE_BUILDER_NAME,
+    )
+
+
+class TestSlaveScannerScan(TestCaseWithFactory):
     """Tests `SlaveScanner.scan` method.
 
     This method uses the old framework for scanning and dispatching builds.
@@ -83,15 +90,31 @@
         test_publisher.setUpDefaultDistroSeries(hoary)
         test_publisher.addFakeChroots()
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def _resetBuilder(self, builder):
         """Reset the given builder and its job."""
-
         builder.builderok = True
         job = builder.currentjob
         if job is not None:
             job.reset()
 
-        transaction.commit()
+    def getFreshBuilder(self, slave=None, name=BOB_THE_BUILDER_NAME,
+                        failure_count=0):
+        """Return a builder.
+
+        The builder is taken from sample data, but reset to a usable state.
+        Be careful: this is not a proper factory method.  Identical calls
+        return (and reset) the same builder.  Don't rely on that though;
+        maybe someday we'll have a proper factory here.
+        """
+        if slave is None:
+            slave = OkSlave()
+        builder = getUtility(IBuilderSet)[name]
+        self._resetBuilder(builder)
+        builder.setSlaveForTesting(slave)
+        builder.failure_count = failure_count
+        return builder
 
     def assertBuildingJob(self, job, builder, logtail=None):
         """Assert the given job is building on the given builder."""
@@ -107,14 +130,14 @@
         self.assertEqual(build.status, BuildStatus.BUILDING)
         self.assertEqual(job.logtail, logtail)
 
-    def _getScanner(self, builder_name=None):
+    def _getScanner(self, builder_name=None, clock=None):
         """Instantiate a SlaveScanner object.
 
         Replace its default logging handler by a testing version.
         """
         if builder_name is None:
             builder_name = BOB_THE_BUILDER_NAME
-        scanner = SlaveScanner(builder_name, BufferLogger())
+        scanner = SlaveScanner(builder_name, BufferLogger(), clock=clock)
         scanner.logger.name = 'slave-scanner'
 
         return scanner
@@ -130,16 +153,13 @@
     def testScanDispatchForResetBuilder(self):
         # A job gets dispatched to the sampledata builder after it's reset.
 
-        # Reset sampledata builder.
-        builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
-        self._resetBuilder(builder)
-        builder.setSlaveForTesting(OkSlave())
-        # Set this to 1 here so that _checkDispatch can make sure it's
-        # reset to 0 after a successful dispatch.
-        builder.failure_count = 1
+        # Obtain a builder.   Initialize failure count to 1 so that
+        # _checkDispatch can make sure that a successful dispatch resets
+        # the count to 0.
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.getFreshBuilder(failure_count=1)
 
         # Run 'scan' and check its result.
-        self.layer.txn.commit()
         self.layer.switchDbUser(config.builddmaster.dbuser)
         scanner = self._getScanner()
         d = defer.maybeDeferred(scanner.scan)
@@ -153,28 +173,26 @@
         to the asynchonous dispatcher and the builder remained active
         and IDLE.
         """
-        self.assertTrue(slave is None, "Unexpected slave.")
+        self.assertIs(None, slave, "Unexpected slave.")
 
         builder = getUtility(IBuilderSet).get(builder.id)
         self.assertTrue(builder.builderok)
-        self.assertTrue(builder.currentjob is None)
+        self.assertIs(None, builder.currentjob)
 
     def testNoDispatchForMissingChroots(self):
         # When a required chroot is not present the `scan` method
         # should not return any `RecordingSlaves` to be processed
         # and the builder used should remain active and IDLE.
-
-        # Reset sampledata builder.
-        builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
-        self._resetBuilder(builder)
-
-        # Remove hoary/i386 chroot.
-        login('foo.bar@canonical.com')
-        ubuntu = getUtility(IDistributionSet).getByName('ubuntu')
-        hoary = ubuntu.getSeries('hoary')
-        pocket_chroot = hoary.getDistroArchSeries('i386').getPocketChroot()
-        removeSecurityProxy(pocket_chroot).chroot = None
-        transaction.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.getFreshBuilder()
+            # Remove hoary/i386 chroot.
+            login('foo.bar@canonical.com')
+            ubuntu = getUtility(IDistributionSet).getByName('ubuntu')
+            hoary = ubuntu.getSeries('hoary')
+            pocket_chroot = (
+                hoary.getDistroArchSeries('i386').getPocketChroot())
+            removeSecurityProxy(pocket_chroot).chroot = None
+
         login(ANONYMOUS)
 
         # Run 'scan' and check its result.
@@ -214,8 +232,8 @@
 
         # Disable the sampledata builder
         login('foo.bar@canonical.com')
-        builder.builderok = False
-        transaction.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder.builderok = False
         login(ANONYMOUS)
 
         # Run 'scan' and check its result.
@@ -245,9 +263,9 @@
         builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
 
         login('foo.bar@canonical.com')
-        builder.builderok = True
-        builder.setSlaveForTesting(BuildingSlave(build_id='8-1'))
-        transaction.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder.builderok = True
+            builder.setSlaveForTesting(BuildingSlave(build_id='8-1'))
         login(ANONYMOUS)
 
         job = builder.currentjob
@@ -261,19 +279,18 @@
         return d
 
     def test_scan_with_nothing_to_dispatch(self):
-        factory = LaunchpadObjectFactory()
-        builder = factory.makeBuilder()
-        builder.setSlaveForTesting(OkSlave())
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.factory.makeBuilder()
+            builder.setSlaveForTesting(OkSlave())
         scanner = self._getScanner(builder_name=builder.name)
         d = scanner.scan()
         return d.addCallback(self._checkNoDispatch, builder)
 
     def test_scan_with_manual_builder(self):
         # Reset sampledata builder.
-        builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
-        self._resetBuilder(builder)
-        builder.setSlaveForTesting(OkSlave())
-        builder.manual = True
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.getFreshBuilder()
+            builder.manual = True
         scanner = self._getScanner()
         d = scanner.scan()
         d.addCallback(self._checkNoDispatch, builder)
@@ -281,10 +298,9 @@
 
     def test_scan_with_not_ok_builder(self):
         # Reset sampledata builder.
-        builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
-        self._resetBuilder(builder)
-        builder.setSlaveForTesting(OkSlave())
-        builder.builderok = False
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.getFreshBuilder()
+            builder.builderok = False
         scanner = self._getScanner()
         d = scanner.scan()
         # Because the builder is not ok, we can't use _checkNoDispatch.
@@ -293,33 +309,35 @@
         return d
 
     def test_scan_of_broken_slave(self):
-        builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
-        self._resetBuilder(builder)
-        builder.setSlaveForTesting(BrokenSlave())
-        builder.failure_count = 0
+        with BuilddManagerTestFixture.extraSetUp():
+            builder = self.getFreshBuilder(slave=BrokenSlave())
         scanner = self._getScanner(builder_name=builder.name)
         d = scanner.scan()
         return assert_fails_with(d, xmlrpclib.Fault)
 
     def _assertFailureCounting(self, builder_count, job_count,
                                expected_builder_count, expected_job_count):
+        # Avoid circular imports.
+        from lp.buildmaster import manager as manager_module
+
         # If scan() fails with an exception, failure_counts should be
         # incremented.  What we do with the results of the failure
         # counts is tested below separately, this test just makes sure that
         # scan() is setting the counts.
         def failing_scan():
             return defer.fail(Exception("fake exception"))
-        scanner = self._getScanner()
-        scanner.scan = failing_scan
-        from lp.buildmaster import manager as manager_module
-        self.patch(manager_module, 'assessFailureCounts', FakeMethod())
-        builder = getUtility(IBuilderSet)[scanner.builder_name]
-
-        builder.failure_count = builder_count
-        builder.currentjob.specific_job.build.failure_count = job_count
-        # The _scanFailed() calls abort, so make sure our existing
-        # failure counts are persisted.
-        self.layer.txn.commit()
+
+        with BuilddManagerTestFixture.extraSetUp():
+            scanner = self._getScanner()
+            scanner.scan = failing_scan
+            self.patch(manager_module, 'assessFailureCounts', FakeMethod())
+            builder = getUtility(IBuilderSet)[scanner.builder_name]
+
+            builder.failure_count = builder_count
+            builder.currentjob.specific_job.build.failure_count = job_count
+            # The _scanFailed() calls abort, so make sure our existing failure
+            # counts are persisted by exiting the extraSetUp() context (which
+            # commits).
 
         # singleCycle() calls scan() which is our fake one that throws an
         # exception.
@@ -365,9 +383,9 @@
         scanner = self._getScanner()
         scanner.scan = failing_scan
         builder = getUtility(IBuilderSet)[scanner.builder_name]
-        builder.failure_count = Builder.FAILURE_THRESHOLD
-        builder.currentjob.reset()
-        self.layer.txn.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder.failure_count = Builder.FAILURE_THRESHOLD
+            builder.currentjob.reset()
 
         d = scanner.singleCycle()
 
@@ -388,17 +406,19 @@
         # Reset sampledata builder.
         builder = removeSecurityProxy(
             getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME])
-        self._resetBuilder(builder)
-        self.assertEqual(0, builder.failure_count)
-        builder.setSlaveForTesting(slave)
-        builder.vm_host = "fake_vm_host"
+        with BuilddManagerTestFixture.extraSetUp():
+            self._resetBuilder(builder)
+            self.assertEqual(0, builder.failure_count)
+            builder.setSlaveForTesting(slave)
+            builder.vm_host = "fake_vm_host"
 
         scanner = self._getScanner()
 
         # Get the next job that will be dispatched.
         job = removeSecurityProxy(builder._findBuildCandidate())
-        job.virtualized = True
-        builder.virtualized = True
+        with BuilddManagerTestFixture.extraSetUp():
+            job.virtualized = True
+            builder.virtualized = True
         d = scanner.singleCycle()
 
         def check(ignored):
@@ -430,19 +450,20 @@
         # Set the sample data builder building with the slave from above.
         builder = getUtility(IBuilderSet)[BOB_THE_BUILDER_NAME]
         login('foo.bar@canonical.com')
-        builder.builderok = True
-        # For now, we can only cancel virtual builds.
-        builder.virtualized = True
-        builder.vm_host = "fake_vm_host"
-        builder.setSlaveForTesting(slave)
-        transaction.commit()
+        with BuilddManagerTestFixture.extraSetUp():
+            builder.builderok = True
+            # For now, we can only cancel virtual builds.
+            builder.virtualized = True
+            builder.vm_host = "fake_vm_host"
+            builder.setSlaveForTesting(slave)
         login(ANONYMOUS)
         buildqueue = builder.currentjob
         self.assertBuildingJob(buildqueue, builder)
 
         # Now set the build to CANCELLING.
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(buildqueue)
-        build.status = BuildStatus.CANCELLING
+        with BuilddManagerTestFixture.extraSetUp():
+            build.status = BuildStatus.CANCELLING
 
         # Run 'scan' and check its results.
         self.layer.switchDbUser(config.builddmaster.dbuser)
@@ -459,6 +480,63 @@
         d.addCallback(check_cancelled, builder, buildqueue)
         return d
 
+    def makeFakeFailure(self):
+        """Produce a fake failure for use with SlaveScanner._scanFailed."""
+        FakeFailure = namedtuple('FakeFailure', ['getErrorMessage', 'check'])
+        return FakeFailure(
+            FakeMethod(self.factory.getUniqueString()),
+            FakeMethod(True))
+
+    def test_interleaved_success_and_failure_do_not_interfere(self):
+        # It's possible for one builder to fail while another continues
+        # to function properly.  When that happens, the failed builder
+        # may cause database changes to be rolled back.  But that does
+        # not affect the functioning builder.
+        clock = task.Clock()
+
+        with BuilddManagerTestFixture.extraSetUp():
+            broken_builder = self.getFreshBuilder(
+                slave=BrokenSlave(), name=BOB_THE_BUILDER_NAME)
+            broken_scanner = self._getScanner(
+                builder_name=broken_builder.name)
+            good_builder = self.getFreshBuilder(
+                slave=WaitingSlave(), name=FROG_THE_BUILDER_NAME)
+            good_build = self.factory.makeBinaryPackageBuild(
+                distroarchseries=self.factory.makeDistroArchSeries())
+
+            # The good build is being handled by the good builder.
+            buildqueue = good_build.queueBuild()
+            buildqueue.builder = good_builder
+
+            removeSecurityProxy(
+                good_build.build_farm_job).date_started = UTC_NOW
+
+        # The good builder requests information from a successful build,
+        # and up receiving it, updates the build's metadata.
+        # Our dependencies string goes into the build, and its
+        # date_finished will be set.
+        dependencies = self.factory.getUniqueString()
+        PackageBuild.storeBuildInfo(
+            good_build, None, {'dependencies': dependencies})
+        clock.advance(1)
+
+        # The broken scanner experiences a failure before the good
+        # scanner is receiving its data.  This aborts the ongoing
+        # transaction.
+        # As a somewhat weird example, if the builder changed its own
+        # title, that change will be rolled back.
+        original_broken_builder_title = broken_builder.title
+        broken_builder.title = self.factory.getUniqueString()
+        broken_scanner._scanFailed(self.makeFakeFailure())
+
+        # The work done by the good scanner is retained.  The
+        # storeBuildInfo code committed it.
+        self.assertEqual(dependencies, good_build.dependencies)
+        self.assertIsNot(None, good_build.date_finished)
+
+        # The work done by the broken scanner is rolled back.
+        self.assertEqual(original_broken_builder_title, broken_builder.title)
+
 
 class TestCancellationChecking(TestCaseWithFactory):
     """Unit tests for the checkCancellation method."""
@@ -475,9 +553,12 @@
         self.scanner.builder = self.builder
         self.scanner.logger.name = 'slave-scanner'
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def test_ignores_nonvirtual(self):
         # If the builder is nonvirtual make sure we return False.
-        self.builder.virtualized = False
+        with BuilddManagerTestFixture.extraSetUp():
+            self.builder.virtualized = False
         d = self.scanner.checkCancellation(self.builder)
         return d.addCallback(self.assertFalse)
 
@@ -485,7 +566,8 @@
         # If the builder has no buildqueue associated,
         # make sure we return False.
         buildqueue = self.builder.currentjob
-        buildqueue.reset()
+        with BuilddManagerTestFixture.extraSetUp():
+            buildqueue.reset()
         d = self.scanner.checkCancellation(self.builder)
         return d.addCallback(self.assertFalse)
 
@@ -493,7 +575,8 @@
         # If the active build is not in a CANCELLING state, ignore it.
         buildqueue = self.builder.currentjob
         build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(buildqueue)
-        build.status = BuildStatus.BUILDING
+        with BuilddManagerTestFixture.extraSetUp():
+            build.status = BuildStatus.BUILDING
         d = self.scanner.checkCancellation(self.builder)
         return d.addCallback(self.assertFalse)
 
@@ -507,11 +590,14 @@
             return defer.succeed((None, None, 0))
         slave = OkSlave()
         slave.resume = fake_resume
-        self.builder.vm_host = "fake_vm_host"
-        self.builder.setSlaveForTesting(slave)
-        buildqueue = self.builder.currentjob
-        build = getUtility(IBinaryPackageBuildSet).getByQueueEntry(buildqueue)
-        build.status = BuildStatus.CANCELLING
+
+        with BuilddManagerTestFixture.extraSetUp():
+            self.builder.vm_host = "fake_vm_host"
+            self.builder.setSlaveForTesting(slave)
+            buildqueue = self.builder.currentjob
+            build = getUtility(
+                IBinaryPackageBuildSet).getByQueueEntry(buildqueue)
+            build.status = BuildStatus.CANCELLING
 
         def check(result):
             self.assertEqual(1, call_counter.call_count)

=== modified file 'lib/lp/buildmaster/tests/test_packagebuild.py'
--- lib/lp/buildmaster/tests/test_packagebuild.py	2011-12-30 06:14:56 +0000
+++ lib/lp/buildmaster/tests/test_packagebuild.py	2012-01-03 12:12:17 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Tests for `IPackageBuild`."""
@@ -12,6 +12,7 @@
 import tempfile
 
 from storm.store import Store
+from testtools.deferredruntest import AsynchronousDeferredRunTest
 from zope.component import getUtility
 from zope.security.interfaces import Unauthorized
 from zope.security.proxy import removeSecurityProxy
@@ -26,8 +27,10 @@
     IPackageBuildSet,
     IPackageBuildSource,
     )
+from lp.buildmaster.model.builder import BuilderSlave
 from lp.buildmaster.model.buildfarmjob import BuildFarmJob
 from lp.buildmaster.model.packagebuild import PackageBuild
+from lp.buildmaster.testing import BuilddManagerTestFixture
 from lp.buildmaster.tests.mock_slaves import WaitingSlave
 from lp.registry.interfaces.pocket import PackagePublishingPocket
 from lp.services.config import config
@@ -281,12 +284,10 @@
 
 
 class TestHandleStatusMixin:
-    """Tests for `IPackageBuild`s handleStatus method.
-
-    This should be run with a Trial TestCase.
-    """
+    """Tests for `IPackageBuild`s handleStatus method."""
 
     layer = LaunchpadZopelessLayer
+    run_tests_with = AsynchronousDeferredRunTest.make_factory(timeout=20)
 
     def makeBuild(self):
         """Allow classes to override the build with which the test runs."""
@@ -303,7 +304,7 @@
         self.build.buildqueue_record.setDateStarted(UTC_NOW)
         self.slave = WaitingSlave('BuildStatus.OK')
         self.slave.valid_file_hashes.append('test_file_hash')
-        builder.setSlaveForTesting(self.slave)
+        self.patch(BuilderSlave, 'makeBuilderSlave', FakeMethod(self.slave))
 
         # We overwrite the buildmaster root to use a temp directory.
         tempdir = tempfile.mkdtemp()
@@ -321,6 +322,8 @@
         removeSecurityProxy(self.build).verifySuccessfulUpload = FakeMethod(
             result=True)
 
+        self.useFixture(BuilddManagerTestFixture())
+
     def assertResultCount(self, count, result):
         self.assertEquals(
             1, len(os.listdir(os.path.join(self.upload_root, result))))
@@ -344,7 +347,7 @@
         def got_status(ignored):
             self.assertEqual(BuildStatus.FAILEDTOUPLOAD, self.build.status)
             self.assertResultCount(0, "failed")
-            self.assertIdentical(None, self.build.buildqueue_record)
+            self.assertIs(None, self.build.buildqueue_record)
 
         d = self.build.handleStatus('OK', None, {
             'filemap': {'/tmp/myfile.py': 'test_file_hash'},
@@ -365,7 +368,8 @@
 
     def test_handleStatus_OK_sets_build_log(self):
         # The build log is set during handleStatus.
-        removeSecurityProxy(self.build).log = None
+        with BuilddManagerTestFixture.extraSetUp():
+            removeSecurityProxy(self.build).log = None
         self.assertEqual(None, self.build.log)
         d = self.build.handleStatus('OK', None, {
                 'filemap': {'myfile.py': 'test_file_hash'},
@@ -386,14 +390,10 @@
 
         def got_status(ignored):
             if expected_notification:
-                self.failIf(
-                    len(pop_notifications()) == 0,
-                    "No notifications received")
+                self.assertNotEqual(
+                    0, len(pop_notifications()), "No notifications received.")
             else:
-                self.failIf(
-                    len(pop_notifications()) > 0,
-                    "Notifications received")
-
+                self.assertContentEqual([], pop_notifications())
         d = self.build.handleStatus(status, None, {})
         return d.addCallback(got_status)
 
@@ -408,7 +408,9 @@
 
     def test_date_finished_set(self):
         # The date finished is updated during handleStatus_OK.
-        removeSecurityProxy(self.build).date_finished = None
+        with BuilddManagerTestFixture.extraSetUp():
+            removeSecurityProxy(self.build).date_finished = None
+
         self.assertEqual(None, self.build.date_finished)
         d = self.build.handleStatus('OK', None, {
                 'filemap': {'myfile.py': 'test_file_hash'},

=== modified file 'lib/lp/code/model/tests/test_sourcepackagerecipebuild.py'
--- lib/lp/code/model/tests/test_sourcepackagerecipebuild.py	2012-01-01 02:58:52 +0000
+++ lib/lp/code/model/tests/test_sourcepackagerecipebuild.py	2012-01-03 12:11:57 +0000
@@ -13,14 +13,15 @@
 
 from pytz import utc
 from storm.locals import Store
+from testtools.deferredruntest import AsynchronousDeferredRunTest
 import transaction
-from twisted.trial.unittest import TestCase as TrialTestCase
 from zope.component import getUtility
 from zope.security.proxy import removeSecurityProxy
 
 from lp.app.errors import NotFoundError
 from lp.buildmaster.enums import BuildStatus
 from lp.buildmaster.interfaces.buildqueue import IBuildQueue
+from lp.buildmaster.model.builder import BuilderSlave
 from lp.buildmaster.model.buildfarmjob import BuildFarmJob
 from lp.buildmaster.model.packagebuild import PackageBuild
 from lp.buildmaster.tests.mock_slaves import WaitingSlave
@@ -588,14 +589,11 @@
         self.assertEquals(0, len(notifications))
 
 
-class TestBuildNotifications(TrialTestCase):
+class TestBuildNotifications(TestCaseWithFactory):
 
     layer = LaunchpadZopelessLayer
 
-    def setUp(self):
-        super(TestBuildNotifications, self).setUp()
-        from lp.testing.factory import LaunchpadObjectFactory
-        self.factory = LaunchpadObjectFactory()
+    run_tests_with = AsynchronousDeferredRunTest.make_factory(timeout=20)
 
     def prepare_build(self, fake_successful_upload=False):
         queue_record = self.factory.makeSourcePackageRecipeBuildJob()
@@ -608,7 +606,7 @@
                 result=True)
         queue_record.builder = self.factory.makeBuilder()
         slave = WaitingSlave('BuildStatus.OK')
-        queue_record.builder.setSlaveForTesting(slave)
+        self.patch(BuilderSlave, 'makeBuilderSlave', FakeMethod(slave))
         return build
 
     def assertDeferredNotifyCount(self, status, build, expected_count):
@@ -666,5 +664,5 @@
 
 
 class TestHandleStatusForSPRBuild(
-    MakeSPRecipeBuildMixin, TestHandleStatusMixin, TrialTestCase):
+    MakeSPRecipeBuildMixin, TestHandleStatusMixin, TestCaseWithFactory):
     """IPackageBuild.handleStatus works with SPRecipe builds."""

=== modified file 'lib/lp/services/database/tests/test_transaction_policy.py'
--- lib/lp/services/database/tests/test_transaction_policy.py	2012-01-01 02:58:52 +0000
+++ lib/lp/services/database/tests/test_transaction_policy.py	2012-01-03 12:17:29 +0000
@@ -41,6 +41,15 @@
         self.factory.makePerson(name=name)
         return name
 
+    def writeToDatabaseAndCommit(self):
+        """Write an object to the database and commit.
+
+        :return: A token that `hasDatabaseBeenWrittenTo` can look for.
+        """
+        name = self.writeToDatabase()
+        transaction.commit()
+        return name
+
     def hasDatabaseBeenWrittenTo(self, test_token):
         """Is the object made by `writeToDatabase` present in the database?
 
@@ -76,16 +85,26 @@
 
         self.assertRaises(InternalError, make_forbidden_update)
 
-    def test_will_not_start_in_ongoing_transaction(self):
-        # You cannot enter a DatabaseTransactionPolicy while already in
-        # a transaction.
+    def test_will_not_go_read_only_when_read_write_transaction_ongoing(self):
+        # You cannot enter a read-only DatabaseTransactionPolicy while in an
+        # active read-write transaction.
         def enter_policy():
-            with DatabaseTransactionPolicy():
+            with DatabaseTransactionPolicy(read_only=True):
                 pass
 
         self.writeToDatabase()
         self.assertRaises(TransactionInProgress, enter_policy)
 
+    def test_will_go_read_write_when_read_write_transaction_ongoing(self):
+        # You can enter a read-write DatabaseTransactionPolicy while in an
+        # active read-write transaction.
+        def enter_policy():
+            with DatabaseTransactionPolicy(read_only=False):
+                pass
+
+        self.writeToDatabase()
+        enter_policy()  # No exception.
+
     def test_successful_exit_requires_commit_or_abort(self):
         # If a read-write policy exits normally (which would probably
         # indicate successful completion of its code), it requires that
@@ -206,28 +225,36 @@
     def test_policy_can_span_transactions(self):
         # It's okay to commit within a policy; the policy will still
         # apply to the next transaction inside the same policy.
-        def write_and_commit():
-            self.writeToDatabase()
-            transaction.commit()
-
         test_token = self.writeToDatabase()
         transaction.commit()
 
         with DatabaseTransactionPolicy(read_only=True):
             self.hasDatabaseBeenWrittenTo(test_token)
             transaction.commit()
-            self.assertRaises(InternalError, write_and_commit)
+            self.assertRaises(InternalError, self.writeToDatabaseAndCommit)
             transaction.abort()
 
     def test_policy_survives_abort(self):
         # Even after aborting the initial transaction, a transaction
         # policy still applies.
-        def write_and_commit():
-            self.writeToDatabase()
-            transaction.commit()
-
         with DatabaseTransactionPolicy(read_only=True):
             self.readFromDatabase()
             transaction.abort()
-            self.assertRaises(InternalError, write_and_commit)
+            self.assertRaises(InternalError, self.writeToDatabaseAndCommit)
             transaction.abort()
+
+    def test_readOnly(self):
+        # DatabaseTransactionPolicy.readOnly() is a function decorator that
+        # applies a read-only policy for the duration of the function call.
+        writeToDatabaseAndCommit = DatabaseTransactionPolicy.readOnly(
+            self.writeToDatabaseAndCommit)
+        with DatabaseTransactionPolicy(read_only=False):
+            self.assertRaises(InternalError, writeToDatabaseAndCommit)
+
+    def test_readWrite(self):
+        # DatabaseTransactionPolicy.readWrite() is a function decorator that
+        # applies a read-write policy for the duration of the function call.
+        writeToDatabaseAndCommit = DatabaseTransactionPolicy.readWrite(
+            self.writeToDatabaseAndCommit)
+        with DatabaseTransactionPolicy(read_only=True):
+            writeToDatabaseAndCommit()  # No exception.

=== modified file 'lib/lp/services/database/transaction_policy.py'
--- lib/lp/services/database/transaction_policy.py	2011-12-30 06:14:56 +0000
+++ lib/lp/services/database/transaction_policy.py	2012-01-03 12:17:29 +0000
@@ -8,6 +8,8 @@
     'DatabaseTransactionPolicy',
     ]
 
+from functools import wraps
+
 from psycopg2.extensions import TRANSACTION_STATUS_IDLE
 import transaction
 from zope.component import getUtility
@@ -92,9 +94,18 @@
 
         :raise TransactionInProgress: if a transaction was already ongoing.
         """
-        self._checkNoTransaction(
-            "Entered DatabaseTransactionPolicy while in a transaction.")
+        # We must check the transaction status before checking the current
+        # policy because getting the policy causes a status change.
+        in_transaction = self._isInTransaction()
         self.previous_policy = self._getCurrentPolicy()
+        # If the current transaction is read-write and we're moving to
+        # read-only then we should check for a transaction. If the current
+        # policy is read-only then we don't care if a transaction is in
+        # progress.
+        if in_transaction and self.read_only and not self.previous_policy:
+            raise TransactionInProgress(
+                "Attempting to enter a read-only transaction while holding "
+                "open a read-write transaction.")
         self._setPolicy(self.read_only)
         # Commit should include the policy itself.  If this breaks
         # because the transaction was already in a failed state before
@@ -133,8 +144,11 @@
     def _isInTransaction(self):
         """Is our store currently in a transaction?"""
         pg_connection = self.store._connection._raw_connection
-        status = pg_connection.get_transaction_status()
-        return status != TRANSACTION_STATUS_IDLE
+        if pg_connection is None:
+            return False
+        else:
+            status = pg_connection.get_transaction_status()
+            return status != TRANSACTION_STATUS_IDLE
 
     def _checkNoTransaction(self, error_msg):
         """Verify that no transaction is ongoing.
@@ -183,3 +197,19 @@
         """
         self.store.execute(
             "SET %s TO %s" % (self.db_switch, quote(read_only)))
+
+    @classmethod
+    def readOnly(cls, func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            with cls(read_only=True):
+                return func(*args, **kwargs)
+        return wrapper
+
+    @classmethod
+    def readWrite(cls, func):
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            with cls(read_only=False):
+                return func(*args, **kwargs)
+        return wrapper

=== modified file 'lib/lp/soyuz/tests/test_binarypackagebuild.py'
--- lib/lp/soyuz/tests/test_binarypackagebuild.py	2012-01-01 02:58:52 +0000
+++ lib/lp/soyuz/tests/test_binarypackagebuild.py	2012-01-03 12:12:17 +0000
@@ -1,4 +1,4 @@
-# Copyright 2009-2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2009-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Test Build features."""
@@ -10,7 +10,6 @@
 
 import pytz
 from storm.store import Store
-from twisted.trial.unittest import TestCase as TrialTestCase
 from zope.component import getUtility
 from zope.security.proxy import removeSecurityProxy
 
@@ -18,6 +17,7 @@
 from lp.buildmaster.interfaces.builder import IBuilderSet
 from lp.buildmaster.interfaces.buildqueue import IBuildQueue
 from lp.buildmaster.interfaces.packagebuild import IPackageBuild
+from lp.buildmaster.model.builder import BuilderSlave
 from lp.buildmaster.model.buildqueue import BuildQueue
 from lp.buildmaster.tests.mock_slaves import WaitingSlave
 from lp.buildmaster.tests.test_packagebuild import (
@@ -48,6 +48,7 @@
     logout,
     TestCaseWithFactory,
     )
+from lp.testing.fakemethod import FakeMethod
 from lp.testing.layers import (
     DatabaseFunctionalLayer,
     LaunchpadZopelessLayer,
@@ -522,7 +523,9 @@
         self.build = gedit_src_hist.createMissingBuilds()[0]
 
         self.builder = self.factory.makeBuilder()
-        self.builder.setSlaveForTesting(WaitingSlave('BuildStatus.OK'))
+        self.patch(
+            BuilderSlave, 'makeBuilderSlave',
+            FakeMethod(WaitingSlave('BuildStatus.OK')))
         self.build.buildqueue_record.markAsBuilding(self.builder)
 
     def testDependencies(self):
@@ -568,7 +571,7 @@
 
 
 class TestHandleStatusForBinaryPackageBuild(
-    MakeBinaryPackageBuildMixin, TestHandleStatusMixin, TrialTestCase):
+    MakeBinaryPackageBuildMixin, TestHandleStatusMixin, TestCaseWithFactory):
     """IPackageBuild.handleStatus works with binary builds."""
 
 

=== modified file 'lib/lp/translations/model/translationtemplatesbuildbehavior.py'
--- lib/lp/translations/model/translationtemplatesbuildbehavior.py	2012-01-01 02:58:52 +0000
+++ lib/lp/translations/model/translationtemplatesbuildbehavior.py	2012-01-03 12:12:34 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """An `IBuildFarmJobBehavior` for `TranslationTemplatesBuildJob`.
@@ -16,6 +16,7 @@
 import tempfile
 
 import pytz
+import transaction
 from twisted.internet import defer
 from zope.component import getUtility
 from zope.interface import implements
@@ -28,6 +29,7 @@
     )
 from lp.buildmaster.model.buildfarmjobbehavior import BuildFarmJobBehaviorBase
 from lp.registry.interfaces.productseries import IProductSeriesSet
+from lp.services.database.transaction_policy import DatabaseTransactionPolicy
 from lp.translations.interfaces.translationimportqueue import (
     ITranslationImportQueue,
     )
@@ -60,8 +62,12 @@
 
             filemap = {}
 
-            return self._builder.slave.build(
-                cookie, self.build_type, chroot_sha1, filemap, args)
+            with DatabaseTransactionPolicy(read_only=False):
+                d = self._builder.slave.build(
+                    cookie, self.build_type, chroot_sha1, filemap, args)
+                transaction.commit()
+
+            return d
         return d.addCallback(got_cache_file)
 
     def _getChroot(self):
@@ -132,13 +138,16 @@
     def storeBuildInfo(build, queue_item, build_status):
         """See `IPackageBuild`."""
         def got_log(lfa_id):
-            build.build.log = lfa_id
-            build.build.builder = queue_item.builder
-            build.build.date_started = queue_item.date_started
-            # XXX cprov 20060615 bug=120584: Currently buildduration includes
-            # the scanner latency, it should really be asking the slave for
-            # the duration spent building locally.
-            build.build.date_finished = datetime.datetime.now(pytz.UTC)
+            transaction.commit()
+            with DatabaseTransactionPolicy(read_only=False):
+                build.build.log = lfa_id
+                build.build.builder = queue_item.builder
+                build.build.date_started = queue_item.date_started
+                # XXX cprov 20060615 bug=120584: Currently buildduration
+                # includes the scanner latency.  It should really be
+                # asking the slave for the duration spent building locally.
+                build.build.date_finished = datetime.datetime.now(pytz.UTC)
+                transaction.commit()
 
         d = build.getLogFromSlave(build, queue_item)
         return d.addCallback(got_log)
@@ -171,7 +180,9 @@
             # dangerous.
             if filename is None:
                 logger.error("Build produced no tarball.")
-                self.setBuildStatus(BuildStatus.FULLYBUILT)
+                with DatabaseTransactionPolicy(read_only=False):
+                    self.setBuildStatus(BuildStatus.FULLYBUILT)
+                    transaction.commit()
                 return
 
             tarball_file = open(filename)
@@ -181,17 +192,23 @@
                     logger.error("Build produced empty tarball.")
                 else:
                     logger.debug("Uploading translation templates tarball.")
-                    self._uploadTarball(
-                        queue_item.specific_job.branch, tarball, logger)
+                    with DatabaseTransactionPolicy(read_only=False):
+                        self._uploadTarball(
+                            queue_item.specific_job.branch, tarball, logger)
+                        transaction.commit()
                     logger.debug("Upload complete.")
             finally:
-                self.setBuildStatus(BuildStatus.FULLYBUILT)
+                with DatabaseTransactionPolicy(read_only=False):
+                    self.setBuildStatus(BuildStatus.FULLYBUILT)
+                    transaction.commit()
                 tarball_file.close()
                 os.remove(filename)
 
         def build_info_stored(ignored):
             if build_status == 'OK':
-                self.setBuildStatus(BuildStatus.UPLOADING)
+                with DatabaseTransactionPolicy(read_only=False):
+                    self.setBuildStatus(BuildStatus.UPLOADING)
+                    transaction.commit()
                 logger.debug("Processing successful templates build.")
                 filemap = slave_status.get('filemap')
                 d = self._readTarball(queue_item, filemap, logger)
@@ -199,7 +216,10 @@
                 d.addCallback(clean_slave)
                 return d
 
-            self.setBuildStatus(BuildStatus.FAILEDTOBUILD)
+            with DatabaseTransactionPolicy(read_only=False):
+                self.setBuildStatus(BuildStatus.FAILEDTOBUILD)
+                transaction.commit()
+
             return clean_slave(None)
 
         d = self.storeBuildInfo(self, queue_item, build_status)

=== modified file 'lib/lp/translations/tests/test_translationtemplatesbuildbehavior.py'
--- lib/lp/translations/tests/test_translationtemplatesbuildbehavior.py	2011-12-30 01:10:37 +0000
+++ lib/lp/translations/tests/test_translationtemplatesbuildbehavior.py	2012-01-03 12:12:34 +0000
@@ -1,4 +1,4 @@
-# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# Copyright 2010-2011 Canonical Ltd.  This software is licensed under the
 # GNU Affero General Public License version 3 (see the file LICENSE).
 
 """Unit tests for TranslationTemplatesBuildBehavior."""
@@ -20,6 +20,7 @@
     IBuildFarmJobBehavior,
     )
 from lp.buildmaster.interfaces.buildqueue import IBuildQueueSet
+from lp.buildmaster.testing import BuilddManagerTestFixture
 from lp.buildmaster.tests.mock_slaves import (
     SlaveTestHelpers,
     WaitingSlave,
@@ -73,26 +74,29 @@
         Anything that might communicate with build slaves and such
         (which we can't really do here) is mocked up.
         """
-        specific_job = self.factory.makeTranslationTemplatesBuildJob(
-            branch=branch)
-        behavior = IBuildFarmJobBehavior(specific_job)
-        slave = WaitingSlave()
-        behavior._builder = removeSecurityProxy(self.factory.makeBuilder())
-        behavior._builder.setSlaveForTesting(slave)
-        if use_fake_chroot:
-            lf = self.factory.makeLibraryFileAlias()
-            self.layer.txn.commit()
-            behavior._getChroot = lambda: lf
-        return behavior
+        with BuilddManagerTestFixture.extraSetUp():
+            specific_job = self.factory.makeTranslationTemplatesBuildJob(
+                branch=branch)
+            behavior = IBuildFarmJobBehavior(specific_job)
+            slave = WaitingSlave()
+            behavior._builder = removeSecurityProxy(
+                self.factory.makeBuilder())
+            behavior._builder.setSlaveForTesting(slave)
+            if use_fake_chroot:
+                lf = self.factory.makeLibraryFileAlias()
+                self.layer.txn.commit()
+                behavior._getChroot = lambda: lf
+            return behavior
 
     def makeProductSeriesWithBranchForTranslation(self):
-        productseries = self.factory.makeProductSeries()
-        branch = self.factory.makeProductBranch(
-            productseries.product)
-        productseries.branch = branch
-        productseries.translations_autoimport_mode = (
-            TranslationsBranchImportMode.IMPORT_TEMPLATES)
-        return productseries
+        with BuilddManagerTestFixture.extraSetUp():
+            productseries = self.factory.makeProductSeries()
+            branch = self.factory.makeProductBranch(
+                productseries.product)
+            productseries.branch = branch
+            productseries.translations_autoimport_mode = (
+                TranslationsBranchImportMode.IMPORT_TEMPLATES)
+            return productseries
 
 
 class TestTranslationTemplatesBuildBehavior(
@@ -105,6 +109,7 @@
     def setUp(self):
         super(TestTranslationTemplatesBuildBehavior, self).setUp()
         self.slave_helper = self.useFixture(SlaveTestHelpers())
+        self.useFixture(BuilddManagerTestFixture())
 
     def _becomeBuilddMaster(self):
         """Log into the database as the buildd master."""
@@ -145,13 +150,13 @@
         # _getChroot produces the current chroot for the current Ubuntu
         # release, on the nominated architecture for
         # architecture-independent builds.
-        ubuntu = getUtility(ILaunchpadCelebrities).ubuntu
-        current_ubuntu = ubuntu.currentseries
-        distroarchseries = current_ubuntu.nominatedarchindep
-
-        # Set an arbitrary chroot file.
-        fake_chroot_file = getUtility(ILibraryFileAliasSet)[1]
-        distroarchseries.addOrUpdateChroot(fake_chroot_file)
+        with BuilddManagerTestFixture.extraSetUp():
+            ubuntu = getUtility(ILaunchpadCelebrities).ubuntu
+            current_ubuntu = ubuntu.currentseries
+            distroarchseries = current_ubuntu.nominatedarchindep
+            # Set an arbitrary chroot file.
+            fake_chroot_file = getUtility(ILibraryFileAliasSet)[1]
+            distroarchseries.addOrUpdateChroot(fake_chroot_file)
 
         behavior = self.makeBehavior(use_fake_chroot=False)
         chroot = behavior._getChroot()
@@ -180,10 +185,11 @@
 
     def test_updateBuild_WAITING_OK(self):
         # Hopefully, a build will succeed and produce a tarball.
-        behavior = self.makeBehavior()
-        behavior._uploadTarball = FakeMethod()
-        queue_item = FakeBuildQueue(behavior)
-        builder = behavior._builder
+        with BuilddManagerTestFixture.extraSetUp():
+            behavior = self.makeBehavior()
+            behavior._uploadTarball = FakeMethod()
+            queue_item = FakeBuildQueue(behavior)
+            builder = behavior._builder
 
         d = behavior.dispatchBuildToSlave(queue_item, logging)
 
@@ -221,10 +227,12 @@
 
     def test_updateBuild_WAITING_failed(self):
         # Builds may also fail (and produce no tarball).
-        behavior = self.makeBehavior()
-        behavior._uploadTarball = FakeMethod()
-        queue_item = FakeBuildQueue(behavior)
-        builder = behavior._builder
+        with BuilddManagerTestFixture.extraSetUp():
+            behavior = self.makeBehavior()
+            behavior._uploadTarball = FakeMethod()
+            queue_item = FakeBuildQueue(behavior)
+            builder = behavior._builder
+
         d = behavior.dispatchBuildToSlave(queue_item, logging)
 
         def got_dispatch((status, info)):
@@ -264,10 +272,12 @@
     def test_updateBuild_WAITING_notarball(self):
         # Even if the build status is "OK," absence of a tarball will
         # not faze the Behavior class.
-        behavior = self.makeBehavior()
-        behavior._uploadTarball = FakeMethod()
-        queue_item = FakeBuildQueue(behavior)
-        builder = behavior._builder
+        with BuilddManagerTestFixture.extraSetUp():
+            behavior = self.makeBehavior()
+            behavior._uploadTarball = FakeMethod()
+            queue_item = FakeBuildQueue(behavior)
+            builder = behavior._builder
+
         d = behavior.dispatchBuildToSlave(queue_item, logging)
 
         def got_dispatch((status, info)):
@@ -301,11 +311,12 @@
         return d
 
     def test_updateBuild_WAITING_uploads(self):
-        productseries = self.makeProductSeriesWithBranchForTranslation()
-        branch = productseries.branch
-        behavior = self.makeBehavior(branch=branch)
-        queue_item = FakeBuildQueue(behavior)
-        builder = behavior._builder
+        with BuilddManagerTestFixture.extraSetUp():
+            productseries = self.makeProductSeriesWithBranchForTranslation()
+            branch = productseries.branch
+            behavior = self.makeBehavior(branch=branch)
+            queue_item = FakeBuildQueue(behavior)
+            builder = behavior._builder
 
         d = behavior.dispatchBuildToSlave(queue_item, logging)
 

